about summary refs log tree commit homepage
path: root/lib/PublicInbox/ManifestJsGz.pm
diff options
context:
space:
mode:
Diffstat (limited to 'lib/PublicInbox/ManifestJsGz.pm')
-rw-r--r--lib/PublicInbox/ManifestJsGz.pm52
1 files changed, 32 insertions, 20 deletions
diff --git a/lib/PublicInbox/ManifestJsGz.pm b/lib/PublicInbox/ManifestJsGz.pm
index 31cf15dc..be5d5f2a 100644
--- a/lib/PublicInbox/ManifestJsGz.pm
+++ b/lib/PublicInbox/ManifestJsGz.pm
@@ -1,24 +1,22 @@
-# Copyright (C) 2020-2021 all contributors <meta@public-inbox.org>
+# Copyright (C) all contributors <meta@public-inbox.org>
 # License: AGPL-3.0+ <https://www.gnu.org/licenses/agpl-3.0.txt>
 
-# generates manifest.js.gz for grokmirror(1)
+# generates manifest.js.gz for grokmirror(1) via PublicInbox::WWW
+# This doesn't parse manifest.js.gz (that happens in LeiMirror)
 package PublicInbox::ManifestJsGz;
-use strict;
-use v5.10.1;
+use v5.12;
 use parent qw(PublicInbox::WwwListing);
-use bytes (); # length
 use PublicInbox::Config;
 use IO::Compress::Gzip qw(gzip);
 use HTTP::Date qw(time2str);
 
 my $json = PublicInbox::Config::json();
 
-# called by WwwListing
-sub url_regexp {
+sub url_filter {
         my ($ctx) = @_;
         # grokmirror uses relative paths, so it's domain-dependent
-        # SUPER calls PublicInbox::WwwListing::url_regexp
-        $ctx->SUPER::url_regexp('publicInbox.grokManifest', 'match=domain');
+        # SUPER calls PublicInbox::WwwListing::url_filter
+        $ctx->SUPER::url_filter('publicInbox.grokManifest', 'match=domain');
 }
 
 sub inject_entry ($$$;$) {
@@ -29,7 +27,7 @@ sub inject_entry ($$$;$) {
         $ctx->{manifest}->{$url_path} = $ent;
 }
 
-sub manifest_add ($$;$$) {
+sub manifest_add ($$;$$) { # slow path w/o extindex "all" (or per-inbox)
         my ($ctx, $ibx, $epoch, $default_desc) = @_;
         my $url_path = "/$ibx->{name}";
         my $git;
@@ -55,6 +53,7 @@ sub slow_manifest_add ($$) {
                         manifest_add($ctx, $ibx);
                 }
         };
+        warn "E: $@" if $@;
 }
 
 sub eidx_manifest_add ($$$) {
@@ -67,23 +66,37 @@ sub eidx_manifest_add ($$$) {
                 }
         } else {
                 warn "E: `${\$ibx->eidx_key}' not indexed by $ALL->{topdir}\n";
+                # do not use slow path for global manifest since
+                # it can become catastrophically slow.  per-inbox manifest
+                # is not too bad with dozens of epochs, so never fail that:
+                slow_manifest_add($ctx, $ibx) if $ibx == $ctx->{ibx};
+        }
+}
+
+sub response {
+        my ($class, $ctx) = @_;
+        bless $ctx, $class;
+        my ($re, undef) = $ctx->url_filter;
+        $re // return psgi_triple($ctx);
+        my $iter = PublicInbox::ConfigIter->new($ctx->{www}->{pi_cfg},
+                                        $ctx->can('list_match_i'), $re, $ctx);
+        sub {
+                $ctx->{-wcb} = $_[0]; # HTTP server callback
+                ($ctx->{www}->{pi_cfg}->ALL ||
+                                        !$ctx->{env}->{'pi-httpd.async'}) ?
+                                $iter->each_section : $iter->event_step;
         }
 }
 
 sub ibx_entry {
         my ($ctx, $ibx) = @_;
         my $ALL = $ctx->{www}->{pi_cfg}->ALL;
-        if ($ALL) {
-                eidx_manifest_add($ctx, $ALL, $ibx);
-        } else {
+        $ALL ? eidx_manifest_add($ctx, $ALL, $ibx) :
                 slow_manifest_add($ctx, $ibx);
-        }
-        warn "E: $@" if $@;
 }
 
-sub hide_key { 'manifest' }
+sub hide_key { 'manifest' } # for WwwListing->list_match_i
 
-# overrides WwwListing->psgi_triple
 sub psgi_triple {
         my ($ctx) = @_;
         my $abs2urlpath = delete($ctx->{-abs2urlpath}) // {};
@@ -96,13 +109,12 @@ sub psgi_triple {
         gzip(\$manifest => \(my $out));
         [ 200, [ qw(Content-Type application/gzip),
                  'Last-Modified', time2str($ctx->{-mtime}),
-                 'Content-Length', bytes::length($out) ], [ $out ] ]
+                 'Content-Length', length($out) ], [ $out ] ]
 }
 
 sub per_inbox {
         my ($ctx) = @_;
-        # only one inbox, slow is probably OK
-        slow_manifest_add($ctx, $ctx->{ibx});
+        ibx_entry($ctx, $ctx->{ibx});
         psgi_triple($ctx);
 }